@@ -726,6 +726,64 @@ g_mime_part_set_content_byte_array (GMimePart *mime_part, GByteArray *content)
 	mime_part->content = content;
 }
 
+void
+g_mime_part_append_pre_encoded_content (GMimePart              * part,
+                                        const gchar            * content,
+                                        guint                    len,
+                                        GMimePartEncodingType    encoding)
+{
+	gchar * raw;
+	gint old_size = 0;
+	gint max_new_size = 0;
+
+	g_return_if_fail (part != NULL);
+	g_return_if_fail (content != NULL);
+	
+	/* make sure we've got a content byte array */
+	if (part->content == NULL)
+		part->content = g_byte_array_new ();
+
+	/* make sure we've got enough room for the new decoded data */
+	old_size = part->content->len;
+	max_new_size = old_size + len;
+	g_byte_array_set_size (part->content, max_new_size);
+
+	/* write the decoded data */
+	raw = part->content->data + old_size;
+	switch (encoding)
+	{
+		case GMIME_PART_ENCODING_BASE64:
+			len = g_mime_utils_base64_decode_step (content, len, raw, &part->append_state, &part->append_save);
+			g_byte_array_set_size (part->content, old_size+len);
+			break;
+
+		case GMIME_PART_ENCODING_QUOTEDPRINTABLE:
+			len = g_mime_utils_quoted_decode_step (content, len, raw, &part->append_state, &part->append_save);
+			g_byte_array_set_size (part->content, old_size+len);
+			break;
+
+		default:
+			memcpy (raw, content, len);
+
+			/* do some smart 8bit detection */
+			if (encoding == GMIME_PART_ENCODING_DEFAULT)
+			{
+				/* if we were at default before and this is 8bit text, set encoding to 8bit */
+				if (part->encoding==GMIME_PART_ENCODING_DEFAULT
+					&& g_mime_utils_text_is_8bit (raw, len))
+					encoding = GMIME_PART_ENCODING_8BIT;
+
+				/* if we guessed 8bit before but this isn't an 8bit piece, set encoding back to default */
+				else if (part->encoding == GMIME_PART_ENCODING_8BIT)
+					encoding = g_mime_utils_text_is_8bit (raw, len)
+						? GMIME_PART_ENCODING_8BIT
+						: GMIME_PART_ENCODING_DEFAULT;
+			}
+	}
+
+	part->encoding = encoding;
+}
+
 
 /**
  * g_mime_part_set_pre_encoded_content: Set the pre-encoded content of the mime part
