@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,8 @@
 #include <config.h>
 #include <gnome.h>
 
+#include <pan/base-prefs.h>
+
 #include <pan/dialogs/dialogs.h>
 #include <pan/globals.h>
 #include <pan/grouplist.h>
@@ -58,7 +60,7 @@ group_props_apply (GtkWidget *widget, gint page_num, GroupProps *gp)
 	cpch = gtk_entry_get_text (GTK_ENTRY (gp->nickname_entry));
 	if (cpch==NULL || !*cpch || !strcmp (cpch, group->name))
 		cpch = NULL;
-	group_set_string_field (group, &group->readable_name, cpch);
+	replace_gstr (&group->readable_name, g_strdup(cpch));
 
 	/* download directory */
 	cpch = gtk_entry_get_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY(gp->download_dir))));
