@@ -1,6 +1,8 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Author: Charles Kerr <charles@rebelbase.com>
+ *
+ * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,10 +28,6 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <glib.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -38,9 +36,10 @@
 
 #include <glib.h>
 
-#include "debug.h"
-#include "gnksa.h"
-#include "util.h"
+#include <pan/debug.h>
+#include <pan/gnksa.h>
+#include <pan/pan-i18n.h>
+#include <pan/pan-glib-extensions.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -86,10 +85,6 @@
 ******
 *****/
 
-/*****
-******
-*****/
-
 static gboolean
 is_unquoted_char (int ch)
 {
@@ -274,123 +269,6 @@ read_paren_phrase (const char * pch, const char ** end)
 	return TRUE;
 }
 
-
-/*****
-******
-*****/
-
-/*****
-******
-*****/
-
-static int
-gnksa_split_from (const char * from,
-                  char ** alloc_and_setme_address,
-		  char ** alloc_and_setme_realname,
-		  int * addrtype)
-{
-	char * begin;
-	char * end;
-	char * work;
-	char * lparen;
-
-	*alloc_and_setme_address = NULL;
-	*alloc_and_setme_realname = NULL;
-
-	work = g_strdup (from);
-
-	if (is_nonempty_string(work))
-		g_strstrip (work);
-
-	/* empty string */
-	if (!*work)
-	{
-		*addrtype = GNKSA_ADDRTYPE_OLDSTYLE;
-		g_free (work);
-		return GNKSA_LPAREN_MISSING;
-	}
-
-	end = &work [ strlen(work) - 1 ];
-	debug2 (DEBUG_GNKSA, "work: [%s], end: [%s]", work, end);
-	if (*end == '>') /* Charles Kerr <charles@superpimp.org> */
-	{
-		const char * tmp;
-		*addrtype = GNKSA_ADDRTYPE_ROUTE;
-
-		/* get address part */
-		begin = strrchr (work, '<');
-		if (begin == NULL) {
-			g_free (work);
-			return GNKSA_LANGLE_MISSING;
-		}
-
-		/* copy route address from inside the <> brackets */
-		*alloc_and_setme_address = g_strdup (begin+1);
-		*strchr(*alloc_and_setme_address, '>') = '\0';
-
-		/* From: [plain-phrase space] "<" address ">" */
-		*begin = '\0';
-		tmp = work;
-		if ((*tmp) && (!read_plain_phrase(tmp,&tmp)||!read_space(tmp,&tmp))) {
-			g_free (work);
-			return GNKSA_ILLEGAL_PLAIN_PHRASE;
-		}
-
-		/* get realname part */
-		g_strstrip (work);
-		*alloc_and_setme_realname = g_strdup (work);
-	}
-	else if ((lparen = strchr (work,'(')) != NULL) /* charles@superpimp.org (Charles Kerr) */
-	{
-		size_t len;
-		gchar * address;
-		gchar * real;
-		const gchar * end;
-		int val;
-
-		*addrtype = GNKSA_ADDRTYPE_OLDSTYLE;
-
-		/* address part */
-		address = lparen==work ? NULL : g_strndup (work, lparen-work);
-		if (is_nonempty_string (address))
-			g_strstrip (address);
-		val = gnksa_check_address (address);
-		*alloc_and_setme_address = address;
-		if (val) {
-			g_free (work);
-			return val;
-		}
-
-		/* real name part */
-		real = lparen + 1;
-		g_strstrip (real);
-		len = strlen (real);
-		if (real[len-1]!=')') {
-			g_free (work);
-			return GNKSA_RPAREN_MISSING;
-		}
-		real[len-1] = '\0';
-		end = NULL;
-		if (!read_paren_phrase(lparen+1,&end) || end==NULL) {
-			g_free (work);
-			return GNKSA_ILLEGAL_PAREN_PHRASE;
-		}
-		*alloc_and_setme_realname = g_strdup (real);
-	}
-	else if (strchr(work,'@') != NULL) /* charles@superpimp.org */
-	{
-		return gnksa_check_address (*alloc_and_setme_address = work);
-	}
-	else /* who knows what this thing is... */
-	{
-		*alloc_and_setme_realname = work;
-		return GNKSA_LPAREN_MISSING;
-	}
-
-	g_free (work);
-	return GNKSA_OK;
-}
-
 /*****
 ******
 *****/
@@ -444,8 +322,10 @@ gnksa_check_domain (const char * domain)
 {
 	int i;
 	int label_qty;
-	char ** labels;
 	const char * top;
+	const char * pch;
+	const gchar * label = NULL;
+	gint label_len = 0;
 
 	debug1 (DEBUG_GNKSA, "gnksa_check_domain: [%s]", domain);
 
@@ -461,69 +341,51 @@ gnksa_check_domain (const char * domain)
 		return GNKSA_ZERO_LENGTH_LABEL;
 
 	/* count the labels */
-	labels = g_strsplit (domain, ".", -1);
 	label_qty = 0;
-	while (labels[label_qty] != NULL)
+	pch = domain;
+	while (get_next_token_range (pch, '.', &pch, &label, &label_len))
 		++label_qty;
 
 	/* make sure we have more than one label in the domain */
-	if (label_qty < 2) {
-		g_strfreev (labels);
+	if (label_qty < 2)
 		return GNKSA_SINGLE_DOMAIN;
-	}
 
 	/* check for illegal labels */
-	for (i=0; i<label_qty-1; ++i) {
-		const char * label = labels[i];
-		if (strlen(label) > 63) {
-			g_strfreev (labels);
+	pch = domain;
+	for (i=0; i<label_qty-1; ++i)
+	{
+		get_next_token_range (pch, '.', &pch, &label, &label_len);
+		if (label_len > 63)
 			return GNKSA_ILLEGAL_LABEL_LENGTH;
-		}
-		if (label[0]=='-' || label[strlen(label)-1]=='-') {
-			g_strfreev (labels);
+		if (label[0]=='-' || label[label_len-1]=='-')
 			return GNKSA_ILLEGAL_LABEL_HYPHEN;
-		}
 	}
 
-	/* check existence of the toplevel domain */
-	top = labels[label_qty-1];
+	/* last label -- toplevel domain */
+	top = get_next_token_range (pch, '.', &pch, &label, &label_len);
 	debug1 (DEBUG_GNKSA, "top level domain: [%s]", top);
 	switch ((int)strlen(top))
 	{
 		case 1:
 			if (isdigit((int)top[0]))
-			{
-				g_strfreev (labels);
 				return gnksa_check_domain_literal (domain);
-			}
 
 			/* single-letter TLDs dont exist */
-			g_strfreev (labels);
 			return GNKSA_ILLEGAL_DOMAIN;
 			break;
 
 		case 2:
-			if (isdigit((int)top[0])
-				|| isdigit((int)top[1]))
-			{
-				g_strfreev (labels);
+			if (isdigit((int)top[0]) || isdigit((int)top[1]))
 				return gnksa_check_domain_literal (domain);
-			}
 			break;
 		case 3:
-			if (isdigit((int)top[0])
-				|| isdigit((int)top[2])
-				|| isdigit((int)top[3]))
-			{
-				g_strfreev (labels);
+			if (isdigit((int)top[0]) || isdigit((int)top[2]) || isdigit((int)top[3]))
 				return gnksa_check_domain_literal (domain);
-			}
 			break;
 		default:
 			break;
 	}
 
-	g_strfreev (labels);
 	return GNKSA_OK;
 }
 
@@ -534,31 +396,28 @@ gnksa_check_domain (const char * domain)
 static int
 gnksa_check_localpart (const char * localpart)
 {
-	char ** words;
-	int i;
+	gint word_len = 0;
+	const gchar * pch;
+	const gchar * word = NULL;
 
 	/* make sure it's not empty... */
 	if (!is_nonempty_string(localpart))
 		return GNKSA_LOCALPART_MISSING;
 
 	/* break localpart up into its unquoted words */
-	words = g_strsplit (localpart, ".", -1);
-	for (i=0; words[i]!=NULL; ++i)
+	pch = localpart;
+	while (get_next_token_range (pch, '.', &pch, &word, &word_len))
 	{
-		const char * word = words[i];
-		if (!*word) {
-			g_strfreev (words);
+		gint i;
+
+		if (word_len < 1)
 			return GNKSA_ZERO_LENGTH_LOCAL_WORD;
-		}
-		for (; *word; ++word) {
-			if (!is_unquoted_char(*word)) {
-				g_strfreev (words);
+
+		for (i=0; i<word_len; ++i)
+			if (!is_unquoted_char(word[i]))
 				return GNKSA_INVALID_LOCALPART;
-			}
-		}
 	}
 
-	g_strfreev (words);
 	return GNKSA_OK;
 }
 
@@ -566,10 +425,164 @@ gnksa_check_localpart (const char * localpart)
 ******
 *****/
 
+static gint
+gnksa_check_address (const gchar * address)
+{
+	gint retval = GNKSA_OK;
+	gchar * tmp;
+	gchar * begin;
+
+	/* get rid of vacuous case */
+	if (!is_nonempty_string(address))
+		return GNKSA_LOCALPART_MISSING;
+
+	/* check the address */
+	tmp = g_strdup (address);
+	begin = strrchr (address, '@');
+	if (begin==NULL)
+	{
+		if (retval == GNKSA_OK)
+			retval = GNKSA_INVALID_DOMAIN;
+	}
+	else
+	{
+		/* temporarily split string between username & fqdn */
+		*begin++ = '\0';
+		g_strdown (begin);
+
+		/* check the domain */
+		if (retval == GNKSA_OK)
+			retval = gnksa_check_domain (begin);
+
+		/* check the localpart */
+		if (retval == GNKSA_OK)
+			retval = gnksa_check_localpart (address);
+
+		/* restore separator */
+		*--begin = '@';
+	}
+
+	g_free (tmp);
+	return retval;
+}
+
 /*****
 ******
 *****/
 
+static int
+gnksa_split_from (const char   * from,
+                  char        ** alloc_and_setme_address,
+		  char        ** alloc_and_setme_realname,
+		  int          * addrtype,
+		  gboolean       strict)
+{
+	char * begin;
+	char * end;
+	char * work;
+	char * lparen;
+
+	*alloc_and_setme_address = NULL;
+	*alloc_and_setme_realname = NULL;
+
+	work = g_strdup (from);
+
+	if (is_nonempty_string(work))
+		g_strstrip (work);
+
+	/* empty string */
+	if (!*work)
+	{
+		*addrtype = GNKSA_ADDRTYPE_OLDSTYLE;
+		g_free (work);
+		return GNKSA_LPAREN_MISSING;
+	}
+
+	end = &work [ strlen(work) - 1 ];
+	debug2 (DEBUG_GNKSA, "work: [%s], end: [%s]", work, end);
+	if (*end == '>') /* Charles Kerr <charles@rebelbase.com> */
+	{
+		*addrtype = GNKSA_ADDRTYPE_ROUTE;
+
+		/* get address part */
+		begin = strrchr (work, '<');
+		if (begin == NULL) {
+			g_free (work);
+			return GNKSA_LANGLE_MISSING;
+		}
+
+		/* copy route address from inside the <> brackets */
+		*alloc_and_setme_address = g_strdup (begin+1);
+		*strchr(*alloc_and_setme_address, '>') = '\0';
+
+		/* From: [plain-phrase space] "<" address ">" */
+		*begin = '\0';
+		if (strict) {
+			const char * tmp = work;
+			if ((*tmp) && (!read_plain_phrase(tmp,&tmp)||!read_space(tmp,&tmp))) {
+				g_free (work);
+				return GNKSA_ILLEGAL_PLAIN_PHRASE;
+			}
+		}
+
+		/* get realname part */
+		g_strstrip (work);
+		*alloc_and_setme_realname = g_strdup (work);
+	}
+	else if ((lparen = strchr (work,'(')) != NULL) /* charles@rebelbase.com (Charles Kerr) */
+	{
+		size_t len;
+		gchar * address;
+		gchar * real;
+		const gchar * end;
+
+		*addrtype = GNKSA_ADDRTYPE_OLDSTYLE;
+
+		/* address part */
+		address = lparen==work ? NULL : g_strndup (work, lparen-work);
+		if (is_nonempty_string (address))
+			g_strstrip (address);
+		*alloc_and_setme_address = address;
+		if (strict) {
+			gint val = gnksa_check_address (address);
+			if (val) {
+				g_free (work);
+				return val;
+			}
+		}
+
+		/* real name part */
+		real = lparen + 1;
+		g_strstrip (real);
+		len = strlen (real);
+		if (real[len-1]!=')') {
+			g_free (work);
+			return GNKSA_RPAREN_MISSING;
+		}
+		real[len-1] = '\0';
+		end = NULL;
+		if (strict && (!read_paren_phrase(lparen+1,&end) || end==NULL)) {
+			g_free (work);
+			return GNKSA_ILLEGAL_PAREN_PHRASE;
+		}
+		*alloc_and_setme_realname = g_strdup (real);
+	}
+	else if (strchr(work,'@') != NULL) /* charles@rebelbase.com */
+	{
+		*alloc_and_setme_address = work;
+		return strict ? gnksa_check_address(work) : GNKSA_OK;
+	}
+	else /* who knows what this thing is... */
+	{
+		*alloc_and_setme_realname = work;
+		return GNKSA_LPAREN_MISSING;
+	}
+
+	g_free (work);
+	return GNKSA_OK;
+}
+
+
 /*****
 ******
 *****/
@@ -620,46 +633,6 @@ gnksa_split_addresses (const char * from)
 ******
 *****/
 
-gint
-gnksa_check_address (const gchar * address)
-{
-	gint retval = GNKSA_OK;
-	gchar * tmp;
-	gchar * begin;
-
-	/* get rid of vacuous case */
-	if (!is_nonempty_string(address))
-		return GNKSA_LOCALPART_MISSING;
-
-	/* check the address */
-	tmp = g_strdup (address);
-	begin = strrchr (address, '@');
-	if (begin==NULL)
-	{
-		if (retval == GNKSA_OK)
-			retval = GNKSA_INVALID_DOMAIN;
-	}
-	else
-	{
-		/* temporarily split string between username & fqdn*/
-		*begin++ = '\0';
-		g_strdown (begin);
-
-		/* check the domain */
-		if (retval == GNKSA_OK)
-			retval = gnksa_check_domain (begin);
-
-		/* check the localpart */
-		if (retval == GNKSA_OK)
-			retval = gnksa_check_localpart (address);
-
-		/* restore separator */
-		*--begin = '@';
-	}
-
-	g_free (tmp);
-	return retval;
-}
 
 /*****
 ******
@@ -668,7 +641,8 @@ gnksa_check_address (const gchar * address)
 int
 gnksa_do_check_from (const char   * from,
                      char        ** alloc_and_setme_address,
-                     char        ** alloc_and_setme_realname)
+                     char        ** alloc_and_setme_realname,
+		     gboolean       strict)
 {
 	int addrtype = 0;
 	int retval;
@@ -680,7 +654,8 @@ gnksa_do_check_from (const char   * from,
 	retval = gnksa_split_from (from,
 	                           alloc_and_setme_address,
 		         	   alloc_and_setme_realname, 
-			           &addrtype);
+			           &addrtype,
+				   strict);
 
 	address = *alloc_and_setme_address;
 	realname = *alloc_and_setme_realname;
@@ -726,14 +701,14 @@ gnksa_strip_realname (gchar * realname)
 *****/
 
 int
-gnksa_check_from (const char* from)
+gnksa_check_from (const char* from, gboolean strict)
 {
 	int retval;
 	char * address;
 	char * realname;
 
 	address = realname = NULL;
-	retval = gnksa_do_check_from (from, &address, &realname);
+	retval = gnksa_do_check_from (from, &address, &realname, strict);
 	g_free (address);
 	g_free (realname);
 
@@ -811,56 +786,6 @@ gnksa_check_message_id (const char* message_id)
 ******
 *****/
 
-
-const char *
-gnksa_status_text (int status)
-{
-	switch (status)
-	{
-	case GNKSA_OK: return _("Okay");
-	case GNKSA_INTERNAL_ERROR: return _("Internal Error");
-
-	case GNKSA_LANGLE_MISSING: return _("'<' Missing");
-	case GNKSA_RANGLE_MISSING: return _("'>' Missing");
-	case GNKSA_LPAREN_MISSING: return _("'(' Missing");
-	case GNKSA_RPAREN_MISSING: return _("')' Missing");
-	case GNKSA_ATSIGN_MISSING: return _("'@' Missing");
-
-	case GNKSA_SINGLE_DOMAIN: return _("Single Domain");
-	case GNKSA_INVALID_DOMAIN: return _("Invalid Domain");
-	case GNKSA_ILLEGAL_DOMAIN: return _("Illegal Domain");
-	case GNKSA_UNKNOWN_DOMAIN: return _("Unknown Domain");
-	case GNKSA_INVALID_FQDN_CHAR: return _("Invalid char in domain");
-	case GNKSA_ZERO_LENGTH_LABEL: return _("Zero Length Label");
-	case GNKSA_ILLEGAL_LABEL_LENGTH: return _("Illegal label length");
-	case GNKSA_ILLEGAL_LABEL_HYPHEN: return _("Hyphen in label");
-	case GNKSA_ILLEGAL_LABEL_BEGNUM: return _("fully-qualified domain names must not begin with a digit.");
-	case GNKSA_BAD_DOMAIN_LITERAL: return _("Bad domain literal");
-	case GNKSA_LOCAL_DOMAIN_LITERAL: return _("Local domain literal");
-	case GNKSA_RBRACKET_MISSING: return _("Right bracket missing");
-
-	case GNKSA_LOCALPART_MISSING: return _("Local part missing");
-	case GNKSA_INVALID_LOCALPART: return _("Invalid localpart");
-	case GNKSA_ZERO_LENGTH_LOCAL_WORD: return _("Zero length local word");
-
-	case GNKSA_ILLEGAL_UNQUOTED_CHAR:return _("Illegal unquoted character");
-	case GNKSA_ILLEGAL_QUOTED_CHAR: return _("Illegal quoted character");
-	case GNKSA_ILLEGAL_ENCODED_CHAR: return _("Illegal encoded character");
-	case GNKSA_BAD_ENCODE_SYNTAX: return _("Bad encode syntax");
-	case GNKSA_ILLEGAL_PAREN_PHRASE: return _("Illegal parenthesis phrase");
-	case GNKSA_ILLEGAL_PAREN_CHAR: return _("Illegal parenthesis");
-	case GNKSA_INVALID_REALNAME: return _("Invalid real name");
-	case GNKSA_ILLEGAL_PLAIN_PHRASE: return _("Illegal plain phrase");
-	}
-
-	pan_warn_if_reached ();
-	return _("Internal Error");
-};
-
-/*****
-******
-*****/
-
 static char*
 remove_broken_message_ids_from_references (const char * references)
 {
@@ -923,7 +848,7 @@ remove_broken_message_ids_from_references (const char * references)
  * @param untrimmed "references" string
  * @return newly-allocated trimmed "references" string
  */
-static gchar*
+gchar*
 gnksa_trim_references_to_len (const gchar* refs, int cutoff)
 {
 	char * fixed = remove_broken_message_ids_from_references (refs);
@@ -977,7 +902,7 @@ gnksa_trim_references_to_len (const gchar* refs, int cutoff)
  * them in a suitably-reduced alphabet.
  */
 gchar*
-gnksa_generate_message_id (const gchar * author_addr)
+gnksa_generate_message_id (const gchar * domain_name)
 {
 	char * tmp;
 	GString * msgid = g_string_sized_new (256);
@@ -1000,23 +925,9 @@ gnksa_generate_message_id (const gchar * author_addr)
 	g_string_append_c (msgid, '@');
 
 	/* add fqdn to message-id */
-	{
-		gchar * host = get_host_name ();
-		gchar * fqdn = get_fqdn (host);
-		if (is_nonempty_string(fqdn))
-		{
-			g_string_append (msgid, fqdn);
-		}
-		else if (is_nonempty_string(author_addr))
-		{
-			const gchar * pch = strchr (author_addr, '@');
-			if (pch != NULL)
-				g_string_append (msgid, pch+1);
-		}
-		else g_string_append (msgid, "pancantresolveyourdomain.net");
-		g_free (host);
-		g_free (fqdn);
-	}
+	if (!is_nonempty_string(domain_name))
+		domain_name = "nodomain.net";
+	g_string_append (msgid, domain_name);
 
 	/* end with '>' */
 	g_string_append_c (msgid, '>');
@@ -1027,7 +938,7 @@ gnksa_generate_message_id (const gchar * author_addr)
 	return tmp;
 }
 
-gchar*
+static gchar*
 gnksa_trim_references (const gchar* refs)
 {
 	const int GNKSA_CUTOFF = 998; /* gnksa rule 7 */
@@ -1058,224 +969,114 @@ gnksa_generate_references (const gchar * references,
 	return retval;
 }
 
+/***
+****
+****  DATE
+****
+***/
 
-/*****
-******
-*****/
-
-typedef struct 
+/* converts struct tm to time_t, but does not take the local
+   timezone into account unless ``local'' is nonzero.
+   This is lifted from mutt 1.2.4, which is GPL and
+   copyrighted by Michael R. Elkins <me@cs.hmc.edu> */
+static time_t
+mutt_mktime (struct tm *t)
 {
-	int expected_retval;
-	const char * address;
-}
-AddressCheck;
+	time_t g;
 
-typedef struct
-{
-	int len;
-	const char * in;
-	const char * expected_out;
-}
-ReferencesCheck;
-
-void
-gnksa_regression_tests (void)
-{
-	ReferencesCheck references [] = {
-		{
-			998, /*  gnksa cutoff */
-
-			"<gnksa_pan-0.8.0_001@lull.org>"
-			" <gnksa_pan-0.8.0_002@lull.org> <gnksa_pan-0.8.0_003@lull.org>"
-			" <gnksa_pan-0.8.0_004@lull.org> <gnksa_pan-0.8.0_005@lull.org>"
-			" <gnksa_pan-0.8.0_006@lull.org> <gnksa_pan-0.8.0_007@lull.org>"
-			" <gnksa_pan-0.8.0_008@lull.org> <gnksa_pan-0.8.0_009@lull.org>"
-			" <gnksa_pan-0.8.0_010@lull.org> <gnksa_pan-0.8.0_011@lull.org>"
-			" <gnksa_pan-0.8.0_012@lull.org> <gnksa_pan-0.8.0_013@lull.org>"
-			" <gnksa_pan-0.8.0_014@lull.org> <gnksa_pan-0.8.0_015@lull.org>"
-			" <gnksa_pan-0.8.0_016@lull.org> <gnksa_pan-0.8.0_017@lull.org>"
-			" <gnksa_pan-0.8.0_018@lull.org> <gnksa_pan-0.8.0_019@lull.org>"
-			" <gnksa_pan-0.8.0_020@lull.org> <gnksa_pan-0.8.0_021@lull.org>"
-			" <gnksa_pan-0.8.0_022@lull.org> <gnksa_pan-0.8.0_023@lull.org>"
-			" <gnksa_pan-0.8.0_024@lull.org> <gnksa_pan-0.8.0_025@lull.org>"
-			" <gnksa_pan-0.8.0_026@lull.org> <gnksa_pan-0.8.0_027@lull.org>"
-			" <gnksa_pan-0.8.0_028@lull.org> <gnksa_pan-0.8.0_029@lull.org>"
-			" <gnksa_pan-0.8.0_030@lull.org> <gnksa_pan-0.8.0_031@lull.org>"
-	  		" <gnksa_pan-0.8.0_035.12345@lull.org>",
-
-			"<gnksa_pan-0.8.0_001@lull.org>"
-			" <gnksa_pan-0.8.0_002@lull.org> <gnksa_pan-0.8.0_003@lull.org>"
-			" <gnksa_pan-0.8.0_004@lull.org> <gnksa_pan-0.8.0_005@lull.org>"
-			" <gnksa_pan-0.8.0_006@lull.org> <gnksa_pan-0.8.0_007@lull.org>"
-			" <gnksa_pan-0.8.0_008@lull.org> <gnksa_pan-0.8.0_009@lull.org>"
-			" <gnksa_pan-0.8.0_010@lull.org> <gnksa_pan-0.8.0_011@lull.org>"
-			" <gnksa_pan-0.8.0_012@lull.org> <gnksa_pan-0.8.0_013@lull.org>"
-			" <gnksa_pan-0.8.0_014@lull.org> <gnksa_pan-0.8.0_015@lull.org>"
-			" <gnksa_pan-0.8.0_016@lull.org> <gnksa_pan-0.8.0_017@lull.org>"
-			" <gnksa_pan-0.8.0_018@lull.org> <gnksa_pan-0.8.0_019@lull.org>"
-			" <gnksa_pan-0.8.0_020@lull.org> <gnksa_pan-0.8.0_021@lull.org>"
-			" <gnksa_pan-0.8.0_022@lull.org> <gnksa_pan-0.8.0_023@lull.org>"
-			" <gnksa_pan-0.8.0_024@lull.org> <gnksa_pan-0.8.0_025@lull.org>"
-			" <gnksa_pan-0.8.0_026@lull.org> <gnksa_pan-0.8.0_027@lull.org>"
-			" <gnksa_pan-0.8.0_028@lull.org> <gnksa_pan-0.8.0_029@lull.org>"
-			" <gnksa_pan-0.8.0_030@lull.org> <gnksa_pan-0.8.0_031@lull.org>"
-	  		" <gnksa_pan-0.8.0_035.12345@lull.org>"
-		},
-		{
-			998, /*  gnksa cutoff */
-
-			"<gnksa_pan-0.8.0_001@lull.org>"
-			" <gnksa_pan-0.8.1_001@lull.org>"
-			" <gnksa_pan-0.8.0_002@lull.org> <gnksa_pan-0.8.0_003@lull.org>"
-			" <gnksa_pan-0.8.0_004@lull.org> <gnksa_pan-0.8.0_005@lull.org>"
-			" <gnksa_pan-0.8.0_006@lull.org> <gnksa_pan-0.8.0_007@lull.org>"
-			" <gnksa_pan-0.8.0_008@lull.org> <gnksa_pan-0.8.0_009@lull.org>"
-			" <gnksa_pan-0.8.0_010@lull.org> <gnksa_pan-0.8.0_011@lull.org>"
-			" <gnksa_pan-0.8.0_012@lull.org> <gnksa_pan-0.8.0_013@lull.org>"
-			" <gnksa_pan-0.8.0_014@lull.org> <gnksa_pan-0.8.0_015@lull.org>"
-			" <gnksa_pan-0.8.0_016@lull.org> <gnksa_pan-0.8.0_017@lull.org>"
-			" <gnksa_pan-0.8.0_018@lull.org> <gnksa_pan-0.8.0_019@lull.org>"
-			" <gnksa_pan-0.8.0_020@lull.org> <gnksa_pan-0.8.0_021@lull.org>"
-			" <gnksa_pan-0.8.0_022@lull.org> <gnksa_pan-0.8.0_023@lull.org>"
-			" <gnksa_pan-0.8.0_024@lull.org> <gnksa_pan-0.8.0_025@lull.org>"
-			" <gnksa_pan-0.8.0_026@lull.org> <gnksa_pan-0.8.0_027@lull.org>"
-			" <gnksa_pan-0.8.0_028@lull.org> <gnksa_pan-0.8.0_029@lull.org>"
-			" <gnksa_pan-0.8.0_030@lull.org> <gnksa_pan-0.8.0_031@lull.org>"
-	  		" <gnksa_pan-0.8.0_035.12345@lull.org>",
-
-			"<gnksa_pan-0.8.0_001@lull.org>"
-			" <gnksa_pan-0.8.0_002@lull.org> <gnksa_pan-0.8.0_003@lull.org>"
-			" <gnksa_pan-0.8.0_004@lull.org> <gnksa_pan-0.8.0_005@lull.org>"
-			" <gnksa_pan-0.8.0_006@lull.org> <gnksa_pan-0.8.0_007@lull.org>"
-			" <gnksa_pan-0.8.0_008@lull.org> <gnksa_pan-0.8.0_009@lull.org>"
-			" <gnksa_pan-0.8.0_010@lull.org> <gnksa_pan-0.8.0_011@lull.org>"
-			" <gnksa_pan-0.8.0_012@lull.org> <gnksa_pan-0.8.0_013@lull.org>"
-			" <gnksa_pan-0.8.0_014@lull.org> <gnksa_pan-0.8.0_015@lull.org>"
-			" <gnksa_pan-0.8.0_016@lull.org> <gnksa_pan-0.8.0_017@lull.org>"
-			" <gnksa_pan-0.8.0_018@lull.org> <gnksa_pan-0.8.0_019@lull.org>"
-			" <gnksa_pan-0.8.0_020@lull.org> <gnksa_pan-0.8.0_021@lull.org>"
-			" <gnksa_pan-0.8.0_022@lull.org> <gnksa_pan-0.8.0_023@lull.org>"
-			" <gnksa_pan-0.8.0_024@lull.org> <gnksa_pan-0.8.0_025@lull.org>"
-			" <gnksa_pan-0.8.0_026@lull.org> <gnksa_pan-0.8.0_027@lull.org>"
-			" <gnksa_pan-0.8.0_028@lull.org> <gnksa_pan-0.8.0_029@lull.org>"
-			" <gnksa_pan-0.8.0_030@lull.org> <gnksa_pan-0.8.0_031@lull.org>"
-	  		" <gnksa_pan-0.8.0_035.12345@lull.org>"
-		},
-		{
-			1024,
-
-			"<gnksa.01@lull.org> <lull.org> <gnksa.03> <@lull.org>"
-			" <gnksa.05@> <gnksa.06@lull.org> <>"
-			" <gnksa.08@lull.org> <gnksa.09@lull.org"
-			" <gnksa.10@lull.org> gnksa.11@lull.org>"
-			" <gnksa.12@@lull.org> <gnksa.13@lull.org>.14@lull.org>"
-			" gnksa.15@lull.org <gnksa.16@lull.org>",
-
-			"<gnksa.01@lull.org> <gnksa.06@lull.org>"
-			" <gnksa.08@lull.org> <gnksa.10@lull.org>"
-			" <gnksa.13@lull.org>"
-			" <gnksa.16@lull.org>"
-		},
-		{
-			1024,
-
-			"<a@b.a> <d@f.uk> <postmaster@l.uk> <n@o.uk> <@bar.uk>"
-			" <foo@bar.com><baz@xyzzy.org> <foo@> <foo@"
-			" <blah@trala> <blah@trala.org>",
-
-			"<a@b.a> <d@f.uk> <n@o.uk> <foo@bar.com>"
-			" <baz@xyzzy.org> <blah@trala> <blah@trala.org>"
-		},
-		{
-			36,
-			"<a@b.uk> <d@f.uk> <h@i.uk> <k@l.uk> <n@o.uk> <q@r.uk>",
-			"<a@b.uk> <k@l.uk> <n@o.uk> <q@r.uk>"
-		},
-		{
-			35,
-			"<a@b.uk> <d@f.uk> <h@i.uk> <k@l.uk> <n@o.uk> <q@r.uk>",
-			"<a@b.uk> <k@l.uk> <n@o.uk> <q@r.uk>"
-		},
-		{
-			34,
-			"<a@b.uk> <d@f.uk> <h@i.uk> <k@l.uk> <n@o.uk> <q@r.uk>",
-			"<a@b.uk> <n@o.uk> <q@r.uk>"
-		},
-		{
-			32,
-			"<a@b.uk> <d@f.uk> <h@i.uk> <k@l.uk> <n@o.uk> <q@r.uk>",
-			"<a@b.uk> <n@o.uk> <q@r.uk>",
-		},
-		{
-			27,
-			"<a@b.uk> <d@f.uk> <h@i.uk> <k@l.uk> <n@o.uk> <q@r.uk>",
-			"<a@b.uk> <n@o.uk> <q@r.uk>",
-		},
-		{
-			26,
-			"<a@b.uk> <d@f.uk> <h@i.uk> <k@l.uk> <n@o.uk> <q@r.uk>",
-			"<a@b.uk> <n@o.uk> <q@r.uk>",
-		},
-		{
-			25,
-			"<a@b.uk> <d@f.uk> <h@i.uk> <k@l.uk> <n@o.uk> <q@r.uk>",
-			"<a@b.uk> <q@r.uk>",
-		},
+	static int AccumDaysPerMonth[12] = {
+		0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
 	};
 
-	AddressCheck addresses [] = {
-	{ GNKSA_ILLEGAL_PLAIN_PHRASE, "Charles S. Kerr <charles@foo.com>" },
-	{ GNKSA_OK, "\"Charles S. Kerr\" <charles@foo.com>" },
-	{ GNKSA_OK, "charles kerr <charles@superpimp.org>" },
-	{ GNKSA_OK, "Charles \"Pan Programmer\" Kerr <charles@superpimp.org>" },
-	{ GNKSA_OK, "Charles \"Likes, to, put, commas, in, quoted, strings\" Kerr <charles@superpimp.org>" },
-	{ GNKSA_OK, "\"Charles Kerr, Pan Programmer\" <charles@superpimp.org>" },
-	{ GNKSA_ILLEGAL_PLAIN_PHRASE, "Charles Kerr, Pan Programmer <charles@superpimp.org>" },
-	{ GNKSA_INVALID_DOMAIN, "charles kerr <charles>" },
-	{ GNKSA_OK, "charles@superpimp.org" },
-	{ GNKSA_OK, "charles@superpimp.org (Charles Kerr)" },
-	{ GNKSA_SINGLE_DOMAIN, "Charles Kerr <charles@org>" },
-	{ GNKSA_SINGLE_DOMAIN, "Charles Kerr <@org>" },
-	{ GNKSA_OK, "Charles Kerr <charles@[127.0.0.1]>" },
-	{ GNKSA_BAD_DOMAIN_LITERAL, "Charles Kerr <charles@[127..0.1]>" },
-	{ GNKSA_BAD_DOMAIN_LITERAL, "Charles Kerr <charles@[127...1]>" },
-	{ GNKSA_BAD_DOMAIN_LITERAL, "Charles Kerr <charles@[127.0.0.]>" },
-	{ GNKSA_ZERO_LENGTH_LABEL, "<charles@pimp.org.>" },
-	{ GNKSA_ILLEGAL_LABEL_HYPHEN, "<charles@pimp-.org>" },
-	{ GNKSA_OK, "<charles@pimp.org>" },
-	{ GNKSA_OK, "Charles <charles@pimp.asf.fa>" },
-	{ GNKSA_OK, "Charles <charles@pimp.asf.uk>" },
-	{ GNKSA_ZERO_LENGTH_LABEL, "<charles@>" },
-	{ GNKSA_LOCALPART_MISSING, "<@pimp.org>" },
-	{ GNKSA_OK, "Charles Kerr <charles@skywalker.ecn.ou.edu>" },
-	{ GNKSA_LPAREN_MISSING, "Charles Kerr" },
-	{ GNKSA_OK, "looniii@aol.com (Looniii)" },
-	{ GNKSA_OK, "Eric <scare.crow@oz.land>" }
-	};
-	int qty = sizeof(addresses) / sizeof(addresses[0]);
-	int i;
-	for (i=0; i!=qty; ++i)
-	{
-		int retval = gnksa_check_from (addresses[i].address);
+	/* Compute the number of days since January 1 in the same year */
+	g = AccumDaysPerMonth [t->tm_mon % 12];
 
-		regression_test_result_va (
-			retval==addresses[i].expected_retval,
-			"address test", i+1,
-			"address [%s] got [%d]",
-			addresses[i].address, retval);
-	}
+	/* The leap years are 1972 and every 4. year until 2096,
+	 * but this algoritm will fail after year 2099 */
+	g += t->tm_mday;
+	if ((t->tm_year % 4) || t->tm_mon < 2)
+		g--;
+	t->tm_yday = g;
 
-	qty = sizeof(references) / sizeof(references[0]);
-	for (i=0; i!=qty; ++i)
-	{
-		char * pch = gnksa_trim_references_to_len (references[i].in,
-		                                           references[i].len);
-		regression_test_result_va (
-			!strcmp(references[i].expected_out, pch),
-			"references test", i+1,
-                        "expected len [%d], got [%d]",
-			strlen(references[i].expected_out),
-			strlen(pch));
-		g_free (pch);
-	}
+	/* Compute the number of days since January 1, 1970 */
+	g += (t->tm_year - 70) * 365;
+	g += (t->tm_year - 69) / 4;
+
+	/* Compute the number of hours */
+	g *= 24;
+	g += t->tm_hour;
+
+	/* Compute the number of minutes */
+	g *= 60;
+	g += t->tm_min;
+
+	/* Compute the number of seconds */
+	g *= 60;
+	g += t->tm_sec;
+
+	return g;
+}
+
+gchar*
+rfc822_date_generate (time_t secs)
+{
+	static const gchar *const months[] = {
+		"Jan","Feb","Mar","Apr","May","Jun",
+		"Jul","Aug","Sep","Oct","Nov","Dec"
+	};
+	static const gchar *const days[] = {
+		"Sun","Mon","Tue","Wed","Thu","Fri","Sat"
+	};
+
+	int diff;
+	int tz_diff_hours;
+	int tz_diff_minutes;
+	gboolean neg;
+	gchar fmt[64] = { '\0' };
+	gchar buf[64] = { '\0' };
+	struct tm ut;
+	struct tm local;
+
+	/* if 0 passed in, use current time */
+	if (secs == 0)
+		secs = time (NULL);
+
+	/* figure out the difference between this timezone and UT */
+	localtime_r (&secs, &local);
+	gmtime_r (&secs, &ut);
+	diff = (int) difftime (mutt_mktime(&local), mutt_mktime(&ut));
+	neg = diff<0;
+	diff = abs(diff);
+	tz_diff_hours = diff / 3600;
+	diff %= 3600;
+	tz_diff_minutes = diff / 60;
+
+	/* decide on the time string format.  see son of 1036 sec 5.1 for
+	 * the rationale of the offset-from-ut notation.
+         *
+	 * Update: INN 1.7.2-4 (and others) don't accept the <TZ> string at
+	 * the end of son-of-1036's date string.  Thanks to Valentini Jove
+	 * for testing to see what worked & what didn't.
+	 *
+	 * Update 2: Big Thanks to Tov Jacobsen for investigating what
+	 * format other programs use, for suggesting code, and for noting
+	 * that we need to roll our own day-of-week and month abbreviations
+	 * because son-of-1036 requires English text but strftime() localizes
+	 * (Okt instead of Oct or Man instead of Mon).
+	 *
+	 * Update 3: Gaelyne Gasson spotted a bug that the minute tz offset
+	 * should have been divided by 60 to give minutes instead of seconds.
+	 * This was causing problems in that tz, which is +0930.  I'm feeling
+	 * pretty dumb here... :)
+	 */
+	sprintf (fmt, "%s, %%d %s %%Y %%H:%%M:%%S %c%02d%02d",
+		days[local.tm_wday],
+		months[local.tm_mon],
+		(neg?'-':'+'),
+		tz_diff_hours,
+		tz_diff_minutes);
+
+	/* generate the time */
+	strftime (buf, sizeof(buf), fmt, &local);
+	return g_strdup (buf);
 }
