@@ -1,6 +1,8 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Author: Charles Kerr <charles@rebelbase.com>
+ *
+ * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,17 +24,14 @@
 
 #include <string.h>
 
-#include <glib.h>
-#include <gtk/gtk.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-dialog-util.h>
+#include <gnome.h>
 
-#include "debug.h"
-#include "globals.h"
-#include "gui-headers.h"
-#include "prefs.h"
-#include "util.h"
+#include <pan/debug.h>
+#include <pan/globals.h>
+#include <pan/gui-headers.h>
+#include <pan/pan-glib-extensions.h>
+#include <pan/prefs.h>
+#include <pan/util.h>
 
 gulong header_flags = ~0;
 const gint max_header_width = 60;
@@ -48,7 +47,7 @@ showmore_cb (GtkButton *button, gpointer user_data)
 {
 	const gchar* showme = NULL;
 
-	pan_lock();
+	pan_lock ();
 	showme = (const gchar*) gtk_object_get_data ( GTK_OBJECT(button), "text" );
 	gnome_ok_dialog_parented (showme, GTK_WINDOW(Pan.window));
 	pan_unlock();
@@ -211,7 +210,7 @@ gui_headers_set (GtkWidget         * table,
 	/* if message-id turned on, show it unconditionally */
 	if (header_fields & UI_HEADER_MESSAGE_ID)
 	{
-		const gchar * s = article_get_header (article, HEADER_MESSAGE_ID);
+		const gchar * s = article_get_message_id (article);
 		add_header_nolock (table, row++, _("Message-Id: "), s, '\0', max_header_width);
 	}
 
@@ -251,7 +250,7 @@ gui_headers_set (GtkWidget         * table,
 
 	if (header_fields & UI_HEADER_SUBJECT)
 	{
-		add_header_nolock (table, row++, _("Subject: "), article->subject, ' ', max_header_width);
+		add_header_nolock (table, row++, _("Subject: "), article_get_subject(article), ' ', max_header_width);
 	}
 
 	if (need_lock)
