@@ -1,6 +1,8 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Author: Charles Kerr <charles@rebelbase.com>
+ *
+ * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,11 +26,17 @@
 
 #include <config.h>
 
+#include <ctype.h>
+#include <limits.h>
 #include <stdlib.h>
+#include <string.h>
+#include <stdio.h>
 
-#include "debug.h"
-#include "newsrc.h"
-#include "util.h"
+#include <glib.h>
+
+#include <pan/debug.h>
+#include <pan/newsrc.h>
+#include <pan/pan-glib-extensions.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -78,11 +86,12 @@ Range;
 ************/
 
 /*****
-******
+******  INIT READ
 *****/
 
 static void
-parse_read_str (const char * str,
+parse_read_str (Newsrc * n,
+                const char * str,
                 GArray * setme_read)
 {
 	const char * pch = str;
@@ -95,7 +104,10 @@ parse_read_str (const char * str,
 		while (*pch && !error)
 		{
 			gulong high;
-			gulong low = strtol (pch, (char**)&pch, 10);
+			gulong low;
+
+			while (*pch && isspace((int)*pch)) ++pch;
+			low = isdigit((int)*pch) ? strtoul (pch,(char**)&pch,10) : 0;
 			switch (*pch)
 			{
 				case ',' : /* single article */
@@ -107,9 +119,14 @@ parse_read_str (const char * str,
 					break;
 				case '-' : /* start of a range */
 					++pch;
-					high = strtol (pch, (char**)&pch, 10);
-					if (*pch)
-						++pch;
+					while (*pch && isspace((int)*pch)) ++pch;
+					if (!isdigit((int)*pch))
+						high = ULONG_MAX;
+					else {
+						high = strtoul (pch, (char**)&pch, 10);
+						if (*pch == ',')
+							++pch;
+					}
 					break;
 				default: /* some invalid range? */
 					high = low;
@@ -121,19 +138,12 @@ parse_read_str (const char * str,
 
 			if (!error)
 			{
-				Range range;
-				range.low = low;
-				range.high = high;
-				g_array_append_val (setme_read, range);
+				newsrc_mark_range (n, low, high, TRUE);
 			}
 		}
 	}
 }
 
-/*****
-******
-*****/
-
 /**
  * This makes sure that n->read exists.  If it doesn't already exist,
  * it's created based on the information found in n->read_str.
@@ -145,9 +155,13 @@ init_read (Newsrc * n)
 	g_return_if_fail (n->read == NULL);
 
 	n->read = g_array_new (FALSE, FALSE, sizeof(Range));
-	parse_read_str (n->read_str, n->read);
+	parse_read_str (n, n->read_str, n->read);
 }
 
+/*****
+******  RANGE UTIL
+*****/
+
 static int
 compare_pulong_to_pRange (const void * a, const void * b)
 {
@@ -166,113 +180,214 @@ compare_pulong_to_pRange (const void * a, const void * b)
 }
 
 static gboolean
-newsrc_mark_article_read (Newsrc * n, gulong number)
+range_contains (const Range * r1, const Range * r2)
 {
-	int i;
-	GArray * a;
-	gboolean exact_match = FALSE;
-	Range * next = NULL;
-	Range * prev = NULL;
-	gboolean merge_next = FALSE;
-	gboolean merge_prev = FALSE;
-
-	g_return_val_if_fail (n!=NULL, FALSE);
-
-	if (n->read == NULL)
-		init_read (n);
-
-
-	a = n->read;
-	i = lower_bound (&number,
-	                 a->data,
-	                 a->len,
-	                 sizeof(Range),
-	                 compare_pulong_to_pRange,
-	                 &exact_match);
-
-	if (exact_match) /* already read */
-		return TRUE;
-
-	if (i>0) { /* if it's adjacent to prev, merge */
-		prev = &g_array_index (a, Range, i-1);
-		if (prev->high == number-1) {
-			prev->high = number;
-			merge_prev = TRUE;
-		}
-	}
-	if (a->len!=0 && i<a->len) { /* if it's adjacent to next, merge */
-		next = &g_array_index (a, Range, i);
-		if (next->low == number+1) {
-			next->low = number;
-			merge_next = TRUE;
-		}
-	}
-
-	if (merge_prev && merge_next) {
-		next->low = prev->low;
-		g_array_remove_index (a, i-1);
-	}
-
-	/* if it's not absorbed by one of the read
-	   near it, add it as a new range. */
-	if (!merge_prev && !merge_next) {
-		Range r;
-		r.low = r.high = number;
-		g_array_insert_val (a, i, r);
-	}
-
-	return FALSE;
+	return r2->low >= r1->low && r2->high <= r1->high;
 }
 
 static gboolean
-newsrc_mark_article_unread (Newsrc * n, gulong number)
+range_contains_point (const Range * r1, gulong point)
 {
-	int i;
-	GArray * a;
-	gboolean exact_match = FALSE;
-	Range * range;
+	return r1->low<=point && point<=r1->high;
+}
 
-	g_return_val_if_fail (n!=NULL, FALSE);
+/***
+****
+****
+***/
+
+static gboolean
+maybe_merge_ranges (GArray * a, gint low)
+{
+	gboolean merged = FALSE;
+
+	if (0<=low && low<=(int)a->len-2)
+	{
+		Range * r1 = &g_array_index (a, Range, low);
+		Range * r2 = &g_array_index (a, Range, low+1);
+		if (r1->high+1 == r2->low)
+		{
+			r2->low = r1->low;
+			g_array_remove_index (a, low);
+			merged = TRUE;
+		}
+	}
+
+	return merged;
+}
+
+/**
+ * @return the number of articles newly-marked as read 
+ */
+static gulong
+mark_range_read (Newsrc * n, Range * rr)
+{
+	GArray * a = n->read;
+	gint i;
+	gint low_index = 0;
+	gint high_index = 0;
+	gulong retval = 0;
+	gboolean range_found = FALSE;
+
+	low_index = lower_bound (&rr->low,
+	                         a->data,
+	                         a->len,
+	                         sizeof(Range),
+	                         compare_pulong_to_pRange,
+	                         NULL);
+
+	high_index = low_index + lower_bound (&rr->high,
+	                                      (Range*)a->data+low_index,
+	                                      a->len-low_index,
+	                                      sizeof(Range),
+	                                      compare_pulong_to_pRange,
+	                                      NULL);
+
+	for (i=low_index; i<=high_index && i<a->len; )
+	{
+		Range * r = &g_array_index (a, Range, i);
+
+		if (range_contains (rr, r)) /* read range is engulfed; remove */
+		{
+			retval += r->high+1 - r->low;
+			g_array_remove_index (a, i);
+			--high_index;
+		}
+		else if (range_contains (r, rr)) /* no-op */
+		{
+			range_found = TRUE;
+			++i;
+		}
+		else if (range_contains_point (r, rr->high)) /* change low */
+		{
+			range_found = TRUE;
+			retval += r->low - rr->low;
+			r->low = rr->low;
+			if (maybe_merge_ranges (a, i-1))
+				--high_index;
+			else
+				++i;
+		}
+		else if (range_contains_point (r, rr->low)) /* change high */
+		{
+			range_found = TRUE;
+			retval += rr->high - r->high;
+			r->high = rr->high;
+			if (maybe_merge_ranges (a, i))
+				--high_index;
+			else
+				++i;
+		}
+		else ++i;
+	}
+
+	if (!range_found)
+	{
+		Range r = *rr;
+		retval = (r.high+1-r.low) - retval;
+		g_array_insert_val (a, low_index, r);
+		maybe_merge_ranges (a, low_index);
+		maybe_merge_ranges (a, low_index-1);
+	}
+
+	return retval;
+}
+
+static gulong
+mark_range_unread (Newsrc * n, const Range * ur)
+{
+	gint i;
+	gint low_index = -1;
+	gint high_index = -1;
+	gboolean contains_low = FALSE;
+	gboolean contains_high = FALSE;
+	gulong retval = 0;
+	GArray * a = n->read;
+
+	low_index = lower_bound (&ur->low,
+	                         a->data,
+	                         a->len,
+	                         sizeof(Range),
+	                         compare_pulong_to_pRange,
+	                         &contains_low);
+
+	high_index = low_index + lower_bound (&ur->high,
+	                                      (Range*)a->data+low_index,
+	                                      a->len-low_index,
+	                                      sizeof(Range),
+	                                      compare_pulong_to_pRange,
+	                                      &contains_high);
+
+	for (i=low_index; i<=high_index && i<a->len; )
+	{
+		Range * r = &g_array_index (a, Range, i);
+		if (range_contains (ur, r)) /* remove */
+		{
+			retval += (r->high+1) - r->low;
+			g_array_remove_index (a, i);
+			--high_index;
+		}
+		else if (range_contains (r, ur)) /* split */
+		{
+			Range range;
+			range.high = r->high;
+			r->high = ur->low-1;
+			range.low = ur->high+1;
+			retval += ur->high+1-ur->low;
+			g_array_insert_val (a, low_index+1, range);
+			++high_index;
+			i += 2;
+		}
+		else if (range_contains_point (r, ur->low)) /* change high */
+		{
+			retval += r->high+1 - ur->low;
+			r->high = ur->low-1;
+			++i;
+		}
+		else if (range_contains_point (r, ur->high)) /* change low */
+		{
+			retval += ur->high+1 - r->low;
+			r->low = ur->high+1;
+			++i;
+		}
+		else ++i;
+	}
+
+	return retval;
+}
+
+gulong
+newsrc_mark_range (Newsrc * n, gulong low, gulong high, gboolean read)
+{
+	Range r;
+
+	g_return_val_if_fail (n!=NULL, 0);
 
 	if (n->read == NULL)
 		init_read (n);
 
-	a = n->read;
-	i = lower_bound (&number,
-	                 a->data,
-	                 a->len,
-	                 sizeof(Range),
-	                 compare_pulong_to_pRange,
-	                 &exact_match);
+	r.low = MIN(low, high);
+	r.high = MAX(low, high);
 
-	if (!exact_match) /* wasn't read anyway */
-		return FALSE;
-
-	range = &g_array_index (a, Range, i);
-	if (range->low == range->high) /* simple -- just remove the range */
-	{
-		g_array_remove_index (a, i);
-	}
-	else if (number == range->low) /* snip lower bound */
-	{
-		++range->low;
-	}
-	else if (number == range->high) /* snip upper bound */
-	{
-		--range->high;
-	}
-	else /* break the range into two pieces */
-	{
-		Range r;
-		r.low = range->low;
-		r.high = number - 1;
-		range->low = number + 1;
-		g_array_insert_val (a, i, r);
-	}
-
-	return TRUE;
+	return read
+		? mark_range_read (n, &r)
+		: mark_range_unread (n, &r);
 }
 
+gboolean
+newsrc_mark_article (Newsrc * n, gulong number, gboolean read)
+{
+	gulong changed_qty;
+
+	g_return_val_if_fail (n!=NULL, FALSE);
+
+	changed_qty = newsrc_mark_range (n, number, number, read);
+
+	if (read)
+		return changed_qty==0;
+	else /* unread */
+		return changed_qty!=0;
+}
 
 /************
 *************  PROTECTED
@@ -320,6 +435,12 @@ newsrc_init (Newsrc * n, const char * read_str, gulong low, gulong high)
 	n->group_high = high;
 	replace_gstr (&n->read_str, g_strdup (read_str?read_str:""));
 	g_strstrip (n->read_str);
+
+	/* clear out any changes made from a previous run */
+	if (n->read != NULL) {
+		g_array_free (n->read, TRUE);
+		n->read = NULL;
+	}
 }
 
 Newsrc*
@@ -335,12 +456,57 @@ newsrc_new (const char * read_str, gulong low, gulong high)
 *****/
 
 void
-newsrc_set_group_range (Newsrc * e, gulong low, gulong high)
+newsrc_set_group_range (Newsrc * n, gulong low, gulong high)
 {
-	g_return_if_fail (e != NULL);
+	gboolean exact_match;
+	gint low_index;
+	gint high_index;
 
-	e->group_low = low;
-	e->group_high = high;
+	g_return_if_fail (n != NULL);
+
+	n->group_low = low;
+	n->group_high = high;
+
+	/* crop n->read */
+	if (n->read == NULL)
+		init_read (n);
+
+	exact_match = FALSE;
+	low_index = lower_bound (&low,
+		                 n->read->data,
+		                 n->read->len,
+		                 sizeof(Range),
+	 	                 compare_pulong_to_pRange,
+	 	                 &exact_match);
+	if (exact_match) {
+		Range * r = &g_array_index (n->read,Range,low_index);
+		r->low = low;
+	}
+
+	exact_match = FALSE;
+	high_index = lower_bound (&high,
+		                  n->read->data,
+		                  n->read->len,
+		                  sizeof(Range),
+	 	                  compare_pulong_to_pRange,
+	 	                  &exact_match);
+	if (exact_match) {
+		Range * r = &g_array_index (n->read,Range,high_index);
+		r->high = high;
+	} else
+		--high_index;
+
+	if (low_index <= high_index) /*keep the items in the range */
+	{
+		GArray * tmp = g_array_new (FALSE, FALSE, sizeof(Range));
+		g_array_append_vals (tmp, &g_array_index(n->read,Range,low_index), high_index+1-low_index);
+		g_array_free (n->read, TRUE);
+		n->read = tmp;
+	}
+	else /* nothing left */
+	{
+		g_array_set_size (n->read, 0);
+	}
 }
 
 /*****
@@ -383,7 +549,7 @@ newsrc_is_article_read (const Newsrc * n, gulong article_number)
 	             sizeof(Range),
 	             compare_pulong_to_pRange,
 	             &exact_match);
-	
+
 	return exact_match;
 }
 
@@ -391,26 +557,10 @@ newsrc_is_article_read (const Newsrc * n, gulong article_number)
 ******
 *****/
 
-gboolean
-newsrc_mark_article (Newsrc * n, gulong number, gboolean read)
-{
-	gboolean retval;
-
-	g_return_val_if_fail (n!=NULL, FALSE);
-
-	retval = read
-		? newsrc_mark_article_read (n, number)
-		: newsrc_mark_article_unread (n, number);
-
-	return retval;
-}
-
-/*****
-******
-*****/
-
 void
-newsrc_import (Newsrc * n, gboolean * setme_subscribed, const char * import)
+newsrc_import_line (Newsrc      * n,
+                    gboolean    * setme_subscribed,
+                    const char  * import)
 {
 	const char * pch = import;
 	gulong low, high;
@@ -421,7 +571,7 @@ newsrc_import (Newsrc * n, gboolean * setme_subscribed, const char * import)
 	g_return_if_fail (import != NULL);
 
 	/* subscription */
-	while (*pch && *pch!=' ') ++pch;
+	while (*pch && !isspace((int)*pch)) ++pch;
 	--pch;
 	*setme_subscribed = *pch==':';
 
@@ -436,7 +586,7 @@ newsrc_import (Newsrc * n, gboolean * setme_subscribed, const char * import)
 	if (n->read != NULL)
 		g_array_free (n->read, TRUE);
 	a = n->read = g_array_new (FALSE, FALSE, sizeof(Range));
-	parse_read_str (n->read_str, n->read);
+	parse_read_str (n, n->read_str, n->read);
 
 	/* update the low/high based on the imported ranges */
 	low = high = 0;
@@ -458,42 +608,44 @@ newsrc_get_read_str (const Newsrc * n)
 
 	if (n!=NULL && n->read!=NULL) /* someone's been changing the read/unread */
 	{
-		int i;
-		GString * str = g_string_new (NULL);
+		gint i;
+		const gint max_chars_per_range = 22; /* "%lu-%lu," w/10 digits for %lu */
+		gchar * pch = retval = g_new0 (gchar, n->read->len*max_chars_per_range + 1);
 
 		for (i=0; i<n->read->len; ++i)
 		{
-			const Range * range = &g_array_index (n->read, Range, i);
+			const Range * r = &g_array_index (n->read, Range, i);
 
-			/* if this range is too old, don't bother writing it */
-			if (range->high < n->group_low)
-				continue;
+			/* trim to group range */
+			if (r->high < n->group_low) continue;
+			if (r->low > n->group_high) continue;
 
-			g_string_sprintfa (str, "%lu", range->low);
+			pch += sprintf (pch, "%lu", r->low);
 
-			if (range->low != range->high)
-				g_string_sprintfa (str, "-%lu", range->high);
+			if (r->low!=r->high)
+				pch += sprintf (pch, "-%lu", r->high);
 
 			if (i != n->read->len-1)
-				g_string_append_c (str, ',');
+				*pch++ = ',';
 		}
-
-		retval =  str->str;
-		g_string_free (str, FALSE);
 	}
 	else if (n!=NULL && is_nonempty_string(n->read_str)) /* just use the fields passed in */
 	{
 		retval = g_strdup (n->read_str);
 	}
+	else
+	{
+		retval = g_strdup ("");
+	}
 
 	return retval;
 }
 
 void
-newsrc_export (const Newsrc  * n,
-               const char    * group_name,
-               gboolean        subscribed,
-	       GString       * appendme)
+newsrc_export_line (const Newsrc  * n,
+                    const char    * group_name,
+                    gboolean        subscribed,
+	            GString       * appendme)
 {
 	gchar * buf;
 
