@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,11 +26,11 @@
 #include <glib.h>
 #include <libgnome/libgnome.h>
 
-#include "rule.h"
-#include "rule-manager.h"
-#include "rule-manager-p.h"
-#include "file-rules.h"
-#include "util.h"
+#include <pan/rule.h>
+#include <pan/rule-manager.h>
+#include <pan/rule-manager-p.h>
+#include <pan/file-rules.h>
+#include <pan/pan-glib-extensions.h>
 
 /***
 ****
